@@ -1,4 +1,5 @@
 """Declares :class:`Secret`."""
+import asyncio
 import logging
 from collections import defaultdict
 
@@ -17,6 +18,7 @@ class Secret:
     """
     __module__ = 'unimatrix.ext.secrets'
     logger = logging.getLogger('unimatrix.ext.secrets')
+    lock = asyncio.Lock()
 
     @property
     def alias(self) -> str:
@@ -63,14 +65,15 @@ class Secret:
         """Asynchronously retrieve the secret content from its storage
         backend.
         """
-        v = self.__versions[version]
-        if v == NOT_LOADED:
-            self.__versions[version] =\
-                self._encode(await providers.get(self, version))
-            self.logger.debug(
-                "Imported secret (provider: %s, name: %s, alias: %s)",
-                self.provider, self.name, self.alias or self.name
-            )
+        async with self.lock:
+            v = self.__versions[version]
+            if v == NOT_LOADED:
+                self.__versions[version] =\
+                    self._encode(await providers.get(self, version))
+                self.logger.debug(
+                    "Imported secret (provider: %s, name: %s, alias: %s)",
+                    self.provider, self.name, self.alias or self.name
+                )
         return self.__versions[version]
 
     def __getitem__(self, version):
